searchData={"items":[{"type":"task","title":"mix gen_schemas","doc":"Generates Elixir code from Lexicon schema files.","ref":"Mix.Tasks.GenSchemas.html"},{"type":"task","title":"Overview - mix gen_schemas","doc":"- **Complex types** (`object`, `record`) become their own modules with a struct and `@type t`.\n- **Primitive or “inline” types** become `@type`s within a single `ProtoRune.Types` module.\n- **Queries/Procedures** also generate minimal modules, mainly containing `@type input` and `@type output`.\n\nBy default, code is written into `lib/proto_rune`, but you can configure via `--output-dir`.","ref":"Mix.Tasks.GenSchemas.html#module-overview"},{"type":"task","title":"mix gen_schemas.generator","doc":"Handles generation of Elixir code from parsed Lexicon definitions.","ref":"Mix.Tasks.GenSchemas.Generator.html"},{"type":"function","title":"Mix.Tasks.GenSchemas.Generator.generate_code/4","doc":"Generates Elixir code for a definition.","ref":"Mix.Tasks.GenSchemas.Generator.html#generate_code/4"},{"type":"task","title":"mix gen_schemas.parser","doc":"Handles parsing of Lexicon JSON schema files.","ref":"Mix.Tasks.GenSchemas.Parser.html"},{"type":"function","title":"Mix.Tasks.GenSchemas.Parser.build_defs_map/1","doc":"Builds a map of all definitions from the lexicons.","ref":"Mix.Tasks.GenSchemas.Parser.html#build_defs_map/1"},{"type":"function","title":"Mix.Tasks.GenSchemas.Parser.load_lexicons!/1","doc":"Loads and parses all lexicon files from the given path.\nReturns a map of parsed lexicon definitions.","ref":"Mix.Tasks.GenSchemas.Parser.html#load_lexicons!/1"},{"type":"task","title":"mix gen_schemas.type_mapper","doc":"Maps Lexicon types to Elixir type specifications.","ref":"Mix.Tasks.GenSchemas.TypeMapper.html"},{"type":"function","title":"Mix.Tasks.GenSchemas.TypeMapper.to_typespec/3","doc":"Maps a Lexicon type definition to an Elixir type specification.\nRaises with clear error messages for unsupported types.","ref":"Mix.Tasks.GenSchemas.TypeMapper.html#to_typespec/3"},{"type":"behaviour","title":"ProtoRune.Bot","doc":"The `ProtoRune.Bot` module provides the foundational behavior and macros for building bots\nin the ProtoRune ecosystem. It defines the basic structure for bots and ensures that every bot\nadheres to a consistent interface, with customizable event handling, identifier, and password\nretrieval.\n\nThe bot system integrates with the `ProtoRune.Bot.Server` to manage bot lifecycles, handle\nevents, and manage sessions. Bots can use different strategies for receiving notifications,\nsuch as polling or firehose (currently under development).","ref":"ProtoRune.Bot.html"},{"type":"behaviour","title":"Usage - ProtoRune.Bot","doc":"To create a bot using `ProtoRune.Bot`, you need to define your bot module with the required\ncallbacks: `get_identifier/0`, `get_password/0`, and `handle_event/2`.\n\nHere is an example bot implementation:\n\n```elixir\ndefmodule Walle do\n  use ProtoRune.Bot,\n    name: __MODULE__,\n    strategy: :polling\n\n  require Logger\n\n  @impl true\n  def get_identifier, do: System.get_env(\"IDENTIFIER\")\n\n  @impl true\n  def get_password, do: System.get_env(\"PASSWORD\")\n\n  @impl true\n  def handle_event(event, payload) do\n    Logger.info(\"Event: #{event} with URI: #{inspect(payload[:uri])}\")\n  end\nend\n```\n\nIn this example, `Walle` is a bot that uses the polling strategy to fetch notifications.\nIt retrieves its identifier and password from environment variables and logs any events it receives.","ref":"ProtoRune.Bot.html#module-usage"},{"type":"behaviour","title":"Polling Strategy Events - ProtoRune.Bot","doc":"When using the polling strategy, the bot can receive various types of events triggered by\nnotifications from the Bluesky or ATProto services. Each event type corresponds to a specific\nuser action, and a payload containing relevant data is provided. Below is a list of possible\nevents and their associated payloads:","ref":"ProtoRune.Bot.html#module-polling-strategy-events"},{"type":"behaviour","title":"Event Types and Payloads - ProtoRune.Bot","doc":"- **`:reply`**\n  - Triggered when someone replies to a post involving the bot.\n  - **Payload**:\n    - `:uri` - The URI of the post that was replied to.\n    - `:user` - The user who made the reply.\n    - `:content` - The content of the reply post.\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:1234\", user: \"user123\", content: \"Thanks for your post!\"}\n  ```\n\n- **`:quote`**\n  - Triggered when someone quotes the bot's post.\n  - **Payload**:\n    - `:uri` - The URI of the quoted post.\n    - `:user` - The user who quoted the post.\n    - `:content` - The content of the quote.\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:1234\", user: \"user456\", content: \"Great article!\"}\n  ```\n\n- **`:mention`**\n  - Triggered when the bot is mentioned in a post.\n  - **Payload**:\n    - `:uri` - The URI of the post mentioning the bot.\n    - `:user` - The user who mentioned the bot.\n    - `:content` - The content of the post where the bot was mentioned.\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:5678\", user: \"user789\", content: \"Check out @bot's post!\"}\n  ```\n\n- **`:like`**\n  - Triggered when someone likes a post by the bot.\n  - **Payload**:\n    - `:uri` - The URI of the liked post.\n    - `:user` - The user who liked the post.\n    - `:subject` - The subject of the post that was liked (full post data).\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:1234\", user: \"user123\", subject: %{content: \"Nice post!\"}}\n  ```\n\n- **`:repost`**\n  - Triggered when someone reposts content from the bot.\n  - **Payload**:\n    - `:uri` - The URI of the reposted content.\n    - `:user` - The user who reposted the content.\n    - `:post` - The post that was reposted (full post data).\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:5678\", user: \"user987\", post: %{content: \"Check this out!\"}}\n  ```\n\n- **`:follow`**\n  - Triggered when someone follows the bot.\n  - **Payload**:\n    - `:uri` - The URI of the follow event.\n    - `:user` - The user who followed the bot.\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:9876\", user: \"user123\"}\n  ```\n\n- **`:error`**\n  - Triggered when there is an error while processing an event (e.g., failed to fetch a post).\n  - **Payload**:\n    - `:reason` - An atom describing the error.\n\n  Example payload:\n  ```elixir\n  %{reason: {:rate_limited, retry_adter :: integer}}\n  ```","ref":"ProtoRune.Bot.html#module-event-types-and-payloads"},{"type":"behaviour","title":"Callbacks - ProtoRune.Bot","doc":"The following callbacks can be implemented by any bot module that uses `ProtoRune.Bot`:\n\n- `get_identifier/0`: Retrieves the bot's identifier (e.g., username or email). This is used\n  for logging into the service.\n\n- `get_password/0`: Retrieves the bot's password. This is used alongside the identifier\n  to authenticate the bot.\n\n- `handle_event/2`: Handles events that are dispatched to the bot. These events can include\n  mentions, replies, likes, and other interactions that the bot should process.\n\nThe `handle_event/2` function receives:\n- `event`: An atom that represents the type of event (e.g., `:mention`, `:like`, `:reply`).\n- `payload`: A map containing the data related to the event, such as the URI of the post or the user who triggered the event.","ref":"ProtoRune.Bot.html#module-callbacks"},{"type":"behaviour","title":"Optional Callbacks - ProtoRune.Bot","doc":"These callbacks are optional and can be overridden by the bot module:\n\n- `get_identifier/0`: If not implemented, a default error will be raised indicating the callback must be defined.\n- `get_password/0`: Similar to `get_identifier/0`, this must be implemented by the bot if needed for authentication.","ref":"ProtoRune.Bot.html#module-optional-callbacks"},{"type":"behaviour","title":"Bot Lifecycle - ProtoRune.Bot","doc":"The bot is started using `start_link/0`, which initializes the bot server with the provided options.\nThe server handles the bot's session and dispatches messages or events to the bot's defined handlers.\n\nFor instance, starting the bot would look like this:\n\n```elixir\nWalle.start_link()\n```","ref":"ProtoRune.Bot.html#module-bot-lifecycle"},{"type":"behaviour","title":"Customizing the Bot - ProtoRune.Bot","doc":"- **Authentication**: Bots must implement `get_identifier/0` and `get_password/0` to provide authentication details.\n- **Event Handling**: The `handle_event/2` function allows bots to react to different types of events such as mentions, replies, and likes.","ref":"ProtoRune.Bot.html#module-customizing-the-bot"},{"type":"behaviour","title":"Example Workflow - ProtoRune.Bot","doc":"When the bot receives a notification (for example, a new mention), the following happens:\n\n1. The bot's `handle_event/2` callback is called with the event type and payload.\n2. The bot processes the event and can take actions such as replying, liking a post, or logging information.","ref":"ProtoRune.Bot.html#module-example-workflow"},{"type":"behaviour","title":"Notes - ProtoRune.Bot","doc":"- The current implementation supports the polling strategy for fetching notifications. Firehose-based notifications are not yet implemented.\n- Bots should be designed to handle events and messages in a non-blocking manner for efficient performance.","ref":"ProtoRune.Bot.html#module-notes"},{"type":"callback","title":"ProtoRune.Bot.get_identifier/0","doc":"","ref":"ProtoRune.Bot.html#c:get_identifier/0"},{"type":"callback","title":"ProtoRune.Bot.get_password/0","doc":"","ref":"ProtoRune.Bot.html#c:get_password/0"},{"type":"callback","title":"ProtoRune.Bot.handle_event/2","doc":"","ref":"ProtoRune.Bot.html#c:handle_event/2"},{"type":"module","title":"ProtoRune.Bot.Poller","doc":"A GenServer module that handles periodic polling of notifications for a bot, and dispatches\nthese notifications to the appropriate handler functions within the bot.\n\nThe `Poller` connects to the ATProto or Bluesky notification systems and periodically polls\nfor new notifications, processes them, and dispatches them as events to the bot server. It\nhandles various types of notifications including replies, mentions, likes, reposts, and follows.","ref":"ProtoRune.Bot.Poller.html"},{"type":"module","title":"Features - ProtoRune.Bot.Poller","doc":"- Periodic polling of notifications based on a customizable interval.\n- Supports exponential backoff in case of rate limiting or errors.\n- Handles session refresh when required.\n- Dispatches notifications like replies, mentions, quotes, likes, reposts, and follows to the bot.\n- Extensible to handle other types of notifications and custom behavior.","ref":"ProtoRune.Bot.Poller.html#module-features"},{"type":"module","title":"Options - ProtoRune.Bot.Poller","doc":"- `:name` (required) - The name of the GenServer instance.\n- `:interval` (required) - The polling interval in seconds for checking new notifications.\n- `:process_from` - Start polling from a specific date/time.\n- `:last_seen` - The last seen date of notifications.\n- `:cursor` - The cursor to fetch subsequent notifications from the API.\n- `:attempt` - Number of polling attempts, used for backoff.\n- `:server_pid` (required) - The server process that handles events from the poller.\n- `:session` (required) - The session information used to authenticate API requests.","ref":"ProtoRune.Bot.Poller.html#module-options"},{"type":"module","title":"Functions - ProtoRune.Bot.Poller","doc":"- `start_link/1`: Starts the `Poller` process with the given options.\n- `poll_notifications/1`: Fetches the latest notifications from the service and handles them.\n- `handle_notifications/2`: Dispatches each notification to the appropriate event handler in the bot server.\n- `handle_rate_limited/2`: Handles the rate-limiting case by applying exponential backoff before the next poll.\n- `handle_error/2`: Sends error events to the bot server.\n- `dispatch_notification/2`: Dispatches different types of notifications (e.g., replies, quotes, mentions, likes, reposts, follows) to the bot server.","ref":"ProtoRune.Bot.Poller.html#module-functions"},{"type":"module","title":"Example - ProtoRune.Bot.Poller","doc":"You can start the poller like this:\n\n```elixir\nProtoRune.Bot.Poller.start_link([\n  name: :my_bot_poller,\n  interval: 30,\n  session: my_session,\n  server_pid: self()\n])\n```\n\nThe poller will then periodically fetch notifications and dispatch them to the bot server based on the event type.","ref":"ProtoRune.Bot.Poller.html#module-example"},{"type":"module","title":"Backoff Strategy - ProtoRune.Bot.Poller","doc":"The poller implements an exponential backoff strategy when rate-limited or in case of errors.\nThe backoff starts with the defined `interval` and increases exponentially with each failed attempt,\nup to a maximum of 5 minutes.","ref":"ProtoRune.Bot.Poller.html#module-backoff-strategy"},{"type":"module","title":"Internal State - ProtoRune.Bot.Poller","doc":"The `State` struct is used to keep track of:\n- `name`: The name of the poller process.\n- `interval`: The polling interval in seconds.\n- `last_seen`: The last notification timestamp.\n- `cursor`: API cursor for fetching new notifications.\n- `attempt`: The number of failed attempts.\n- `session`: The current session for API requests.\n- `server_pid`: The PID of the server handling the notifications.","ref":"ProtoRune.Bot.Poller.html#module-internal-state"},{"type":"function","title":"ProtoRune.Bot.Poller.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ProtoRune.Bot.Poller.html#child_spec/1"},{"type":"type","title":"ProtoRune.Bot.Poller.kwargs/0","doc":"","ref":"ProtoRune.Bot.Poller.html#t:kwargs/0"},{"type":"type","title":"ProtoRune.Bot.Poller.option/0","doc":"","ref":"ProtoRune.Bot.Poller.html#t:option/0"},{"type":"function","title":"ProtoRune.Bot.Poller.start_link/1","doc":"","ref":"ProtoRune.Bot.Poller.html#start_link/1"},{"type":"module","title":"ProtoRune.Bot.Server","doc":"The `ProtoRune.Bot.Server` module is responsible for managing bot processes in ProtoRune.\nIt handles bot initialization, session management, and event/message dispatching. This\nmodule also integrates with the polling system to retrieve real-time notifications from\nATProto and Bluesky services.\n\nThe bot server can operate in two modes:\n- **Polling**: Periodically fetches notifications using the `ProtoRune.Bot.Poller` module.\n- **Firehose**: (Not yet implemented) Stream real-time events using a websocket-like connection.","ref":"ProtoRune.Bot.Server.html"},{"type":"module","title":"Features - ProtoRune.Bot.Server","doc":"- **Bot Lifecycle Management**: The server manages the entire bot lifecycle, from login\n  and session refresh to handling messages and events.\n- **Polling Strategy**: Supports polling for notifications at regular intervals via the\n  `ProtoRune.Bot.Poller`.\n- **Session Management**: Automatically handles session creation, refresh, and expiration.\n- **Event and Message Handling**: Provides a unified interface for handling events and messages\n  via `handle_message/1` and `handle_event/2`.","ref":"ProtoRune.Bot.Server.html#module-features"},{"type":"module","title":"Options - ProtoRune.Bot.Server","doc":"- `:name` (required) - The name of the bot process.\n- `:lang` - A list of languages the bot supports (default: `[\"en\"]`).\n- `:service` - The service endpoint the bot will connect to (default: `\"https://bsky.social\"`).\n- `:identifier` - The bot's login identifier (e.g., email or username).\n- `:password` - The bot's password for login.\n- `:polling` - Polling configuration (e.g., interval and process_from).\n- `:firehose` - Firehose configuration (not implemented yet).\n- `:strategy` - The bot's strategy for receiving notifications (`:polling` or `:firehose`).","ref":"ProtoRune.Bot.Server.html#module-options"},{"type":"module","title":"Polling Configuration - ProtoRune.Bot.Server","doc":"Polling can be configured with the following options:\n- `:interval` - How often (in seconds) the bot should poll for notifications (default: 5 seconds).\n- `:process_from` - Start processing notifications from a specific timestamp (default: current time).\n\nExample:\n```elixir\nProtoRune.Bot.Server.start_link(\n  name: :my_bot,\n  strategy: :polling,\n  service: \"https://bsky.social\",\n  polling: %{interval: 10}\n)\n```","ref":"ProtoRune.Bot.Server.html#module-polling-configuration"},{"type":"module","title":"Firehose Configuration (Not Implemented) - ProtoRune.Bot.Server","doc":"While not yet available, the firehose strategy will enable real-time notifications using a\nwebsocket connection. Firehose configuration includes:\n- `:relay_uri` - The WebSocket URI for the relay server.\n- `:auto_reconnect` - Automatically reconnect if the connection drops (default: true).\n- `:cursor` - The starting cursor for reading the stream.","ref":"ProtoRune.Bot.Server.html#module-firehose-configuration-not-implemented"},{"type":"module","title":"Functions - ProtoRune.Bot.Server","doc":"- `start_link/1`: Starts the bot process with the given configuration options.\n- `handle_message/2`: Handles incoming messages for the bot.\n- `handle_event/3`: Handles events dispatched to the bot.\n- `format_status/1`: Formats the bot's internal state for debugging.","ref":"ProtoRune.Bot.Server.html#module-functions"},{"type":"module","title":"Session Management - ProtoRune.Bot.Server","doc":"The bot manages its session by authenticating with the ATProto server upon startup.\nIt also refreshes the session token periodically. If the session expires or cannot be\nrefreshed, the bot will stop.","ref":"ProtoRune.Bot.Server.html#module-session-management"},{"type":"module","title":"Example - ProtoRune.Bot.Server","doc":"```elixir\nProtoRune.Bot.Server.start_link([\n  name: :my_bot,\n  strategy: :polling,\n  service: \"https://bsky.social\",\n  identifier: \"my-bot-id\",\n  password: \"super-secret-password\"\n])\n```\n\nThis will start a bot that uses the polling strategy to retrieve notifications from the\nBsky service every 5 seconds.\n\nThe bot can handle messages and events like this:\n\n```elixir\nProtoRune.Bot.Server.handle_message(:my_bot, \"hello\")\nProtoRune.Bot.Server.handle_event(:my_bot, :user_joined, %{user: \"user123\"})\n```","ref":"ProtoRune.Bot.Server.html#module-example"},{"type":"module","title":"Internal State - ProtoRune.Bot.Server","doc":"The server maintains a state that includes:\n- `name`: The bot's name.\n- `service`: The endpoint to connect to.\n- `session`: The session data for making authenticated requests.\n- `poller`: The PID of the polling process (if using the polling strategy).\n- `langs`: The languages the bot supports.","ref":"ProtoRune.Bot.Server.html#module-internal-state"},{"type":"module","title":"Error Handling - ProtoRune.Bot.Server","doc":"- The bot gracefully handles errors such as rate limits and API failures by retrying or\n  stopping the process when necessary.\n- Errors are dispatched as events to the bot, allowing custom error handling.","ref":"ProtoRune.Bot.Server.html#module-error-handling"},{"type":"function","title":"ProtoRune.Bot.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ProtoRune.Bot.Server.html#child_spec/1"},{"type":"type","title":"ProtoRune.Bot.Server.firehose_t/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:firehose_t/0"},{"type":"function","title":"ProtoRune.Bot.Server.firehose_t/1","doc":"","ref":"ProtoRune.Bot.Server.html#firehose_t/1"},{"type":"function","title":"ProtoRune.Bot.Server.firehose_t!/1","doc":"","ref":"ProtoRune.Bot.Server.html#firehose_t!/1"},{"type":"function","title":"ProtoRune.Bot.Server.get_schema/1","doc":"","ref":"ProtoRune.Bot.Server.html#get_schema/1"},{"type":"function","title":"ProtoRune.Bot.Server.handle_event/3","doc":"","ref":"ProtoRune.Bot.Server.html#handle_event/3"},{"type":"function","title":"ProtoRune.Bot.Server.handle_message/2","doc":"","ref":"ProtoRune.Bot.Server.html#handle_message/2"},{"type":"type","title":"ProtoRune.Bot.Server.kwargs/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:kwargs/0"},{"type":"type","title":"ProtoRune.Bot.Server.mapargs/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:mapargs/0"},{"type":"type","title":"ProtoRune.Bot.Server.option/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:option/0"},{"type":"type","title":"ProtoRune.Bot.Server.options_t/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:options_t/0"},{"type":"function","title":"ProtoRune.Bot.Server.options_t/1","doc":"","ref":"ProtoRune.Bot.Server.html#options_t/1"},{"type":"function","title":"ProtoRune.Bot.Server.options_t!/1","doc":"","ref":"ProtoRune.Bot.Server.html#options_t!/1"},{"type":"type","title":"ProtoRune.Bot.Server.polling_t/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:polling_t/0"},{"type":"function","title":"ProtoRune.Bot.Server.polling_t/1","doc":"","ref":"ProtoRune.Bot.Server.html#polling_t/1"},{"type":"function","title":"ProtoRune.Bot.Server.polling_t!/1","doc":"","ref":"ProtoRune.Bot.Server.html#polling_t!/1"},{"type":"function","title":"ProtoRune.Bot.Server.start_link/1","doc":"","ref":"ProtoRune.Bot.Server.html#start_link/1"},{"type":"module","title":"ProtoRune.Lexicon","doc":"Type definitions for the Intermediate Representation (IR) of AT Protocol Lexicons.\nThis structure serves as a bridge between raw lexicon JSON and generated Elixir code.","ref":"ProtoRune.Lexicon.html"},{"type":"type","title":"ProtoRune.Lexicon.array_type/0","doc":"","ref":"ProtoRune.Lexicon.html#t:array_type/0"},{"type":"type","title":"ProtoRune.Lexicon.definition/0","doc":"","ref":"ProtoRune.Lexicon.html#t:definition/0"},{"type":"type","title":"ProtoRune.Lexicon.definition_type/0","doc":"","ref":"ProtoRune.Lexicon.html#t:definition_type/0"},{"type":"type","title":"ProtoRune.Lexicon.error/0","doc":"","ref":"ProtoRune.Lexicon.html#t:error/0"},{"type":"type","title":"ProtoRune.Lexicon.field/0","doc":"","ref":"ProtoRune.Lexicon.html#t:field/0"},{"type":"type","title":"ProtoRune.Lexicon.field_type/0","doc":"","ref":"ProtoRune.Lexicon.html#t:field_type/0"},{"type":"type","title":"ProtoRune.Lexicon.io/0","doc":"","ref":"ProtoRune.Lexicon.html#t:io/0"},{"type":"type","title":"ProtoRune.Lexicon.key_type/0","doc":"","ref":"ProtoRune.Lexicon.html#t:key_type/0"},{"type":"type","title":"ProtoRune.Lexicon.module_path/0","doc":"","ref":"ProtoRune.Lexicon.html#t:module_path/0"},{"type":"type","title":"ProtoRune.Lexicon.number_constraints/0","doc":"","ref":"ProtoRune.Lexicon.html#t:number_constraints/0"},{"type":"type","title":"ProtoRune.Lexicon.object/0","doc":"","ref":"ProtoRune.Lexicon.html#t:object/0"},{"type":"type","title":"ProtoRune.Lexicon.parameters/0","doc":"","ref":"ProtoRune.Lexicon.html#t:parameters/0"},{"type":"type","title":"ProtoRune.Lexicon.primitive_type/0","doc":"","ref":"ProtoRune.Lexicon.html#t:primitive_type/0"},{"type":"type","title":"ProtoRune.Lexicon.procedure/0","doc":"","ref":"ProtoRune.Lexicon.html#t:procedure/0"},{"type":"type","title":"ProtoRune.Lexicon.query/0","doc":"","ref":"ProtoRune.Lexicon.html#t:query/0"},{"type":"type","title":"ProtoRune.Lexicon.record/0","doc":"","ref":"ProtoRune.Lexicon.html#t:record/0"},{"type":"type","title":"ProtoRune.Lexicon.reference_type/0","doc":"","ref":"ProtoRune.Lexicon.html#t:reference_type/0"},{"type":"type","title":"ProtoRune.Lexicon.string_constraints/0","doc":"","ref":"ProtoRune.Lexicon.html#t:string_constraints/0"},{"type":"type","title":"ProtoRune.Lexicon.string_format/0","doc":"","ref":"ProtoRune.Lexicon.html#t:string_format/0"},{"type":"type","title":"ProtoRune.Lexicon.subscription/0","doc":"","ref":"ProtoRune.Lexicon.html#t:subscription/0"},{"type":"type","title":"ProtoRune.Lexicon.t/0","doc":"","ref":"ProtoRune.Lexicon.html#t:t/0"},{"type":"type","title":"ProtoRune.Lexicon.union_type/0","doc":"","ref":"ProtoRune.Lexicon.html#t:union_type/0"},{"type":"module","title":"ProtoRune.Lexicon.Parser","doc":"Parses AT Protocol Lexicon files and converts them into Elixir code structures.\n\nThis module is responsible for:\n- Parsing Lexicon JSON files\n- Validating lexicon structure and types\n- Converting AT Protocol types to Elixir types\n- Creating an intermediate representation for code generation","ref":"ProtoRune.Lexicon.Parser.html"},{"type":"function","title":"ProtoRune.Lexicon.Parser.run/1","doc":"","ref":"ProtoRune.Lexicon.Parser.html#run/1"},{"type":"function","title":"ProtoRune.Lexicon.Parser.run_file/1","doc":"","ref":"ProtoRune.Lexicon.Parser.html#run_file/1"},{"type":"module","title":"ProtoRune.XRPC.Case","doc":"Yeah, in house string casing","ref":"ProtoRune.XRPC.Case.html"},{"type":"function","title":"ProtoRune.XRPC.Case.camelize/1","doc":"","ref":"ProtoRune.XRPC.Case.html#camelize/1"},{"type":"function","title":"ProtoRune.XRPC.Case.snakelize/1","doc":"","ref":"ProtoRune.XRPC.Case.html#snakelize/1"},{"type":"module","title":"ProtoRune.XRPC.Client","doc":"The `XRPC.Client` module handles executing queries and procedures in the XRPC system. It interacts with external services through HTTP requests and performs response validation and schema parsing. The client supports both GET and POST requests, depending on whether the request is a query or a procedure.","ref":"ProtoRune.XRPC.Client.html"},{"type":"module","title":"Overview - ProtoRune.XRPC.Client","doc":"This module allows:\n- **Executing Queries**: Executes `GET` requests for queries.\n- **Executing Procedures**: Executes `POST` requests for procedures with a body.\n- **Error Handling**: Maps various HTTP response codes to custom error messages.","ref":"ProtoRune.XRPC.Client.html#module-overview"},{"type":"module","title":"Functions - ProtoRune.XRPC.Client","doc":"### `execute/1`\n\nExecutes an XRPC query or procedure.\n\n- For **queries**, it performs a `GET` request and validates the query parameters.\n- For **procedures**, it performs a `POST` request and validates the request body.","ref":"ProtoRune.XRPC.Client.html#module-functions"},{"type":"function","title":"ProtoRune.XRPC.Client.apply_case_map/2","doc":"","ref":"ProtoRune.XRPC.Client.html#apply_case_map/2"},{"type":"function","title":"ProtoRune.XRPC.Client.execute/1","doc":"","ref":"ProtoRune.XRPC.Client.html#execute/1"},{"type":"module","title":"ProtoRune.XRPC.DSL","doc":"The `XRPC.DSL` module provides macros to define queries and procedures for interacting with the XRPC system, simplifying the creation of API methods for querying or performing procedures. It supports building custom XRPC queries and procedures by encoding method names and dynamically generating functions based on user-defined parameters.","ref":"ProtoRune.XRPC.DSL.html"},{"type":"module","title":"Overview - ProtoRune.XRPC.DSL","doc":"The primary purpose of this module is to offer a simple DSL for defining queries and procedures in Elixir projects using XRPC. It automates the generation of functions that initiate XRPC requests, reducing boilerplate code for developers.","ref":"ProtoRune.XRPC.DSL.html#module-overview"},{"type":"module","title":"Key Features: - ProtoRune.XRPC.DSL","doc":"- **Query Definition**: Use the `defquery/1` macro to define an XRPC query method.\n- **Procedure Definition**: Use the `defprocedure/2` macro to define an XRPC procedure method.\n- **Parameter Handling**: The `param/2` macro allows specifying parameters and their types for queries and procedures.\n- **Automatic Method Name Encoding**: Converts method names to function names by snakelizing the last segment of the method.","ref":"ProtoRune.XRPC.DSL.html#module-key-features"},{"type":"module","title":"Macros - ProtoRune.XRPC.DSL","doc":"### `defquery/1`\n\nDefines a query function based on the given method name.\n\n```elixir\ndefquery(\"app.bsky.actor.getProfile\")\n```\n\nThis creates a function that returns a new query for the `app.bsky.actor.getProfile` method.\n\n### `defquery/2`\n\nDefines a query with parameters.\n\n```elixir\ndefquery(\"app.bsky.feed.getFeed\", do: block)\n```\n\nWithin the `block`, you can specify parameters using the `param/2` macro. The generated function will include these parameters in the query.\n\n### `defprocedure/2`\n\nDefines a procedure with parameters.\n\n```elixir\ndefprocedure(\"app.bsky.actor.mute\", do: block)\n```\n\nSimilar to `defquery/2`, you can specify parameters in the `block` using the `param/2` macro. This will generate a function that returns a new procedure with the given method and parameters.\n\n### `param/2`\n\nDefines a parameter for queries or procedures.\n\n```elixir\nparam(:actor_id, :string)\n```\n\nThis specifies a parameter with a key of `actor_id` and a type of `:string`, which will be included in the final query or procedure.","ref":"ProtoRune.XRPC.DSL.html#module-macros"},{"type":"module","title":"Usage Example - ProtoRune.XRPC.DSL","doc":"```elixir\ndefmodule MyApp.Bsky do\n  import XRPC.DSL\n\n  defquery \"app.bsky.actor.getProfile\"\n\n  defprocedure \"app.bsky.actor.mute\" do\n    param :actor_id, :string\n  end\nend\n```\n\nIn this example:\n- `get_profile/0` is generated as a function that creates a query for `app.bsky.actor.getProfile`.\n- `mute/0` is generated as a function that creates a procedure for `app.bsky.actor.mute` with the parameter `:actor_id` of type `:string`.","ref":"ProtoRune.XRPC.DSL.html#module-usage-example"},{"type":"macro","title":"ProtoRune.XRPC.DSL.defprocedure/2","doc":"","ref":"ProtoRune.XRPC.DSL.html#defprocedure/2"},{"type":"macro","title":"ProtoRune.XRPC.DSL.defprocedure/3","doc":"","ref":"ProtoRune.XRPC.DSL.html#defprocedure/3"},{"type":"macro","title":"ProtoRune.XRPC.DSL.defquery/2","doc":"","ref":"ProtoRune.XRPC.DSL.html#defquery/2"},{"type":"macro","title":"ProtoRune.XRPC.DSL.defquery/3","doc":"","ref":"ProtoRune.XRPC.DSL.html#defquery/3"},{"type":"function","title":"ProtoRune.XRPC.DSL.encode_method_name/1","doc":"","ref":"ProtoRune.XRPC.DSL.html#encode_method_name/1"},{"type":"type","title":"ProtoRune.XRPC.DSL.options/0","doc":"","ref":"ProtoRune.XRPC.DSL.html#t:options/0"},{"type":"macro","title":"ProtoRune.XRPC.DSL.param/2","doc":"","ref":"ProtoRune.XRPC.DSL.html#param/2"},{"type":"module","title":"ProtoRune.XRPC.Procedure","doc":"The `XRPC.Procedure` module represents a procedure in the XRPC system, encapsulating the method name, request body, and a parser for validating the body. It provides functions to create and manipulate procedure structures.","ref":"ProtoRune.XRPC.Procedure.html"},{"type":"module","title":"Overview - ProtoRune.XRPC.Procedure","doc":"This module allows for:\n- **Creating Procedures**: Use the `new/2` function to create a new procedure with a method and a parser.\n- **Adding a Body**: Use the `put_body/2` function to attach a validated body to the procedure.\n- **String Representation**: The procedure can be converted to a string that represents its full URL.","ref":"ProtoRune.XRPC.Procedure.html#module-overview"},{"type":"module","title":"Functions - ProtoRune.XRPC.Procedure","doc":"### `new/2`\n\nCreates a new procedure with a given method and parser.\n\n```elixir\nXRPC.Procedure.new(\"app.bsky.actor.mute\", from: MyParser)\n```\n\n### `put_body/2`\n\nAttaches a validated body to the procedure.\n\n```elixir\n{:ok, updated_proc} = XRPC.Procedure.put_body(proc, %{\"actor_id\" => \"123\"})\n```\n\nValidates the body using the specified parser and updates the procedure via [peri](https://hexdocs.pm/peri).","ref":"ProtoRune.XRPC.Procedure.html#module-functions"},{"type":"function","title":"ProtoRune.XRPC.Procedure.new/1","doc":"","ref":"ProtoRune.XRPC.Procedure.html#new/1"},{"type":"function","title":"ProtoRune.XRPC.Procedure.new/2","doc":"","ref":"ProtoRune.XRPC.Procedure.html#new/2"},{"type":"function","title":"ProtoRune.XRPC.Procedure.put_body/2","doc":"","ref":"ProtoRune.XRPC.Procedure.html#put_body/2"},{"type":"function","title":"ProtoRune.XRPC.Procedure.put_header/3","doc":"","ref":"ProtoRune.XRPC.Procedure.html#put_header/3"},{"type":"module","title":"ProtoRune.XRPC.Query","doc":"The `XRPC.Query` module is responsible for defining and managing queries in the XRPC system. It encapsulates the method, parameters, headers, and an optional parser, providing functions to create and manipulate query structures.","ref":"ProtoRune.XRPC.Query.html"},{"type":"module","title":"Overview - ProtoRune.XRPC.Query","doc":"This module allows for:\n- **Creating Queries**: Use `new/1` or `new/2` to create a query with an optional parser.\n- **Adding Parameters and Headers**: Use `put_param/3` and `put_header/3` to add query parameters and headers.\n- **String Representation**: Converts a query to a string URL, including parameters if present.","ref":"ProtoRune.XRPC.Query.html#module-overview"},{"type":"module","title":"Functions - ProtoRune.XRPC.Query","doc":"### `new/1`\n\nCreates a new query with the given method.\n\n```elixir\nXRPC.Query.new(\"app.bsky.actor.getProfile\")\n```\n\n### `new/2`\n\nCreates a new query with a method and a parser for validation.\n\n```elixir\nXRPC.Query.new(\"app.bsky.feed.getFeed\", from: MyParser)\n```\n\n### `put_param/3`\n\nAdds or updates a query parameter.\n\n```elixir\nquery = XRPC.Query.put_param(query, :actor_id, \"123\")\n```\n\n### `put_header/3`\n\nAdds or updates a request header.\n\n```elixir\nquery = XRPC.Query.put_header(query, \"Authorization\", \"Bearer token\")\n```","ref":"ProtoRune.XRPC.Query.html#module-functions"},{"type":"function","title":"ProtoRune.XRPC.Query.add_params/2","doc":"","ref":"ProtoRune.XRPC.Query.html#add_params/2"},{"type":"function","title":"ProtoRune.XRPC.Query.new/1","doc":"","ref":"ProtoRune.XRPC.Query.html#new/1"},{"type":"function","title":"ProtoRune.XRPC.Query.new/2","doc":"","ref":"ProtoRune.XRPC.Query.html#new/2"},{"type":"function","title":"ProtoRune.XRPC.Query.put_header/3","doc":"","ref":"ProtoRune.XRPC.Query.html#put_header/3"},{"type":"function","title":"ProtoRune.XRPC.Query.put_param/3","doc":"","ref":"ProtoRune.XRPC.Query.html#put_param/3"},{"type":"protocol","title":"ProtoRune.XRPC.Schema","doc":"","ref":"ProtoRune.XRPC.Schema.html"},{"type":"function","title":"ProtoRune.XRPC.Schema.parse/2","doc":"","ref":"ProtoRune.XRPC.Schema.html#parse/2"},{"type":"type","title":"ProtoRune.XRPC.Schema.t/0","doc":"All the types that implement this protocol.","ref":"ProtoRune.XRPC.Schema.html#t:t/0"},{"type":"extras","title":"Getting Started with ProtoRune","doc":"# Getting Started with ProtoRune","ref":"getting_started.html"},{"type":"extras","title":"Understanding AT Protocol and ProtoRune - Getting Started with ProtoRune","doc":"The AT Protocol (Authenticated Transfer Protocol) is a social networking protocol that emphasizes account portability, algorithmic choice, and scalable interoperability. Unlike traditional social platforms where your data and identity are locked to a single provider, AT Protocol allows users to maintain ownership of their identity and move between providers while preserving their social graph and content.\n\nBluesky is the most well-known application built on AT Protocol, but the protocol itself is designed to support various social applications. ProtoRune provides Elixir developers with tools to build applications, bots, and services that interact with the AT Protocol ecosystem.","ref":"getting_started.html#understanding-at-protocol-and-protorune"},{"type":"extras","title":"Key Concepts - Getting Started with ProtoRune","doc":"**Personal Data Servers (PDS)**\nA PDS hosts user accounts and their data. Users can choose their PDS provider or self-host. ProtoRune connects to these servers to perform operations on behalf of users.\n\n**Decentralized Identity (DID)**\nUsers have persistent identifiers called DIDs that remain stable even when changing providers. These DIDs can be resolved to find the current hosting location of an account. ProtoRune handles DID resolution and validation automatically.\n\n**Records and Repositories**\nUser data in AT Protocol is stored in repositories as typed records (posts, likes, follows, etc.). Each record belongs to a collection and has a specific schema. ProtoRune provides type-safe structs and operations for working with records.\n\n**Lexicons**\nAT Protocol uses Lexicons to define schemas and APIs. ProtoRune generates Elixir code from these Lexicons, providing compile-time type checking and documentation.","ref":"getting_started.html#key-concepts"},{"type":"extras","title":"Getting Started - Getting Started with ProtoRune","doc":"","ref":"getting_started.html#getting-started"},{"type":"extras","title":"Installation - Getting Started with ProtoRune","doc":"Add ProtoRune to your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:proto_rune, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"getting_started.html#installation"},{"type":"extras","title":"Configuration - Getting Started with ProtoRune","doc":"Configure your default service in `config/config.exs`:\n\n```elixir\nconfig :proto_rune, \n  default_service: \"https://bsky.social\",\n  http_client: ProtoRune.HTTPClient.Adapters.Finch\n```","ref":"getting_started.html#configuration"},{"type":"extras","title":"Authentication and Sessions - Getting Started with ProtoRune","doc":"AT Protocol uses sessions for authentication. ProtoRune wraps this in a Session struct:\n\n```elixir\nalias ProtoRune.Session\n\n# Create session with handle (recommended for most users)\n{:ok, session} = ProtoRune.create_session(\"handle.bsky.social\", \"app-password\")\n\n# Or with DID (for more advanced use cases)\n{:ok, session} = ProtoRune.create_session(\"did:plc:1234\", \"app-password\")\n```","ref":"getting_started.html#authentication-and-sessions"},{"type":"extras","title":"Working with Records - Getting Started with ProtoRune","doc":"Records are the core data structures in AT Protocol. Each record type is represented as an Elixir struct with proper typespecs:\n\n```elixir\nalias ProtoRune.Bsky.Post\nalias ProtoRune.Bsky.Profile\n\n# Create and publish a post\npost = Post.new(text: \"Hello from ProtoRune!\")\n{:ok, created} = Post.create(session, post)\n\n# Update a profile\nprofile = Profile.new(\n  display_name: \"Zoey\",\n  description: \"Building with ProtoRune\"\n)\n{:ok, updated} = Profile.update(session, profile)\n```","ref":"getting_started.html#working-with-records"},{"type":"extras","title":"Rich Text Content - Getting Started with ProtoRune","doc":"AT Protocol supports rich text with mentions, links, and formatting. ProtoRune provides both a pipeline API and a sigil for creating rich text:\n\n```elixir\nimport ProtoRune.RichText\n\n# Using the sigil for markdown-like syntax\ntext = ~R\"\"\"\nHello @alice.sky! \nCheck out this #elixir project at [ProtoRune](https://github.com/proto-rune)\n\"\"\"\n\n# Or the pipeline API for programmatic construction\ntext = RichText.new()\n  |> RichText.text(\"Hello \")\n  |> RichText.mention(\"alice.sky\")\n  |> RichText.text(\"! Check out this \")\n  |> RichText.hashtag(\"elixir\")\n  |> RichText.text(\" project at \")\n  |> RichText.link(\"ProtoRune\", \"https://github.com/proto-rune\")\n```","ref":"getting_started.html#rich-text-content"},{"type":"extras","title":"Further Reading - Getting Started with ProtoRune","doc":"- [AT Protocol official documentation](https://atproto.com)\n- [AT Protocol Specifications](https://atproto.com/specs/atp)\n- [Lexicon Reference](https://atproto.com/specs/lexicon)","ref":"getting_started.html#further-reading"},{"type":"extras","title":"Working with Records in ProtoRune","doc":"# Working with Records in ProtoRune\n\nRecords are the fundamental data structures in AT Protocol. They represent everything from posts and profiles to likes and follows. Let's explore how ProtoRune helps you work with records in a type-safe and intuitive way.","ref":"records.html"},{"type":"extras","title":"Understanding Records - Working with Records in ProtoRune","doc":"In AT Protocol, a record is a piece of data that:\n- Belongs to a specific collection (like \"app.bsky.feed.post\")\n- Has a unique record key within that collection\n- Follows a schema defined by a Lexicon\n- Lives in a user's repository\n\nProtoRune represents records as Elixir structs with validation, basically a Ecto.Schema. For example:\n\n> typespecs are abbreviated since it're automatically generated and also for brevity\n\n```elixir\ndefmodule ProtoRune.Bsky.Post do\n  use ProtoRune.Record, collection: \"app.bsky.feed.post\"\n\n  @type t :: %__MODULE__{\n    text: String.t(),\n    created_at: DateTime.t(),\n    langs: [String.t()] | nil,\n    labels: [label()] | nil,\n    embed: embed() | nil,\n    reply_to: reference() | nil,\n    facets: [facet()]\n  }\n\n  embedded_schema do\n    field :text, :string\n    field :langs, {:array, :string}\n    field :created_at, :utc_datetime\n\n    embeds_one :reply_to, Reference\n    embeds_one :embed, Embed\n\n    embeds_many :labels, Label\n    embeds_many :facets, Facet\n  end\nend\n```","ref":"records.html#understanding-records"},{"type":"extras","title":"Record Operations - Working with Records in ProtoRune","doc":"","ref":"records.html#record-operations"},{"type":"extras","title":"Creating Records - Working with Records in ProtoRune","doc":"Every record type provides a `new/1` function for creating instances:\n\n```elixir\nalias ProtoRune.Bsky.Post\nalias ProtoRune.Bsky.Profile\n\n# Create a post instance\npost = Post.new(text: \"Hello world!\")\n# Publish the post\n{:ok, created} = Post.create(session, post)\n\n# Create a profile instance with options\nprofile = Profile.new(\n  display_name: \"Zoey\",\n  description: \"Elixir developer\",\n  avatar: %{\n    image: File.read!(\"avatar.jpg\"),\n    alt: \"Profile picture\"\n  }\n)\n# Create the profile record in the PDS\n{:ok, created} = Profile.create(session, profile)\n```","ref":"records.html#creating-records"},{"type":"extras","title":"Reading Records - Working with Records in ProtoRune","doc":"Records can be fetched by their URI or by collection + record key:\n\n```elixir\n# Get by URI\n{:ok, post} = Post.get(session, \"at://did:plc:1234/app.bsky.feed.post/1234\")\n\n# Get by record key\n{:ok, post} = Post.get(session, did: \"did:plc:1234\", rkey: \"1234\")\n\n# List records from a collection\n{:ok, posts} = Post.list(session, did: \"did:plc:1234\", limit: 50)\n```","ref":"records.html#reading-records"},{"type":"extras","title":"Updating Records - Working with Records in ProtoRune","doc":"Records are immutable in AT Protocol - an update creates a new version:\n\n```elixir\n# Update a profile\n{:ok, _} = Profile.update(session, %{profile | description: \"Updated bio\"})\n```","ref":"records.html#updating-records"},{"type":"extras","title":"Deleting Records - Working with Records in ProtoRune","doc":"```elixir\n# Delete by URI\n{:ok, _} = Post.delete(session, \"at://did:plc:1234/app.bsky.feed.post/1234\")\n\n# Delete by record key\n{:ok, _} = Post.delete(session, rkey: \"1234\")\n```","ref":"records.html#deleting-records"},{"type":"extras","title":"Working with Complex Records - Working with Records in ProtoRune","doc":"","ref":"records.html#working-with-complex-records"},{"type":"extras","title":"Embedded Media - Working with Records in ProtoRune","doc":"Posts can contain embedded media like images:\n\n```elixir\n# Create post with image\nPost.new(text: \"Check out this photo!\")\n|> Post.with_image(binary: File.read!(\"photo.jpg\"), alt: \"A scenic mountain view\")\n|> Post.with_image(binary: File.read!(\"cat.png\"), alt: \"An adorable black cat\")\n|> then(fn post -> Post.create(session, post) end)\n\n# Create post instance with external link, cummulative\nPost.new(text: \"Interesting article\")\n|> Post.with_external_link(\n  uri: \"https://example.com/article\",\n  title: \"Article Title\",\n  description: \"Article description...\"\n)\n|> then(fn post -> Post.create(session, post) end)\n```","ref":"records.html#embedded-media"},{"type":"extras","title":"Reply Threads - Working with Records in ProtoRune","doc":"Posts can be replies to other posts:\n\n```elixir\n# Reply to a post\nPost.new(text: \"Great point!\")\n|> Post.with_reference(root: original_post, parent: parent_post)\n|> then(fn post -> Post.create(session, post) end)\n\n# Get a thread\n{:ok, thread} = Post.get_thread(session, post_uri)\n```","ref":"records.html#reply-threads"},{"type":"extras","title":"Record Validation - Working with Records in ProtoRune","doc":"ProtoRune validates records before sending them to the server:\n\n```elixir\n# Validation happens automatically on create/update\ncase Post.create(session, Post.new(text: 123)) do\n  {:error, %Ecto.Changeset{valid?: false}} ->\n    # Handle validation error\n  {:ok, post} ->\n    # Post created successfully\nend\n\n# Manual validation\ncase Post.validate(post) do\n  {:ok, post} -> # Valid\n  {:error, %Ecto.Changeset{valid?: false}} -> # Invalid\nend\n```","ref":"records.html#record-validation"},{"type":"extras","title":"Custom Record Types - Working with Records in ProtoRune","doc":"You can define your own record types for custom lexicons:\n\n```elixir\ndefmodule MyApp.CustomRecord do\n  use ProtoRune.Record,\n    collection: \"com.example.custom\",\n    lexicon: \"custom_lexicon.json\"\n\n  embedded_schema do\n    field :field_1, :string\n    field :field_2, :map\n  end\n  \n  # Define your schema and functions...\nend\n```","ref":"records.html#custom-record-types"},{"type":"extras","title":"Understanding XRPC and Low-Level API Usage","doc":"# Understanding XRPC and Low-Level API Usage","ref":"xrpc.html"},{"type":"extras","title":"What is XRPC? - Understanding XRPC and Low-Level API Usage","doc":"XRPC (Cross-server Remote Procedure Call) is AT Protocol's approach to HTTP APIs. While it follows RESTful principles, XRPC adds some protocol-specific features:\n\n- **Lexicon-defined endpoints**: Each endpoint is defined by a Lexicon schema\n- **Strongly-typed parameters**: Input and output are validated against schemas\n- **Namespaced methods**: Endpoints follow a hierarchical naming (e.g., `com.atproto.repo.createRecord`)\n- **Session-based auth**: Uses JWT tokens for authentication\n- **Consistent error handling**: Standardized error responses across services","ref":"xrpc.html#what-is-xrpc"},{"type":"extras","title":"XRPC in ProtoRune - Understanding XRPC and Low-Level API Usage","doc":"While ProtoRune provides high-level abstractions like `ProtoRune.create_session/2`, understanding the XRPC layer helps when:\n- Building custom features\n- Working with new Lexicons\n- Debugging issues\n- Implementing advanced functionality\n\nHere's how the layers connect:\n\n```elixir\n{:ok, session} = ProtoRune.create_session(\"identifier\", \"password\")\n\n# High-level API (recommended for most uses)\n{:ok, post} = ProtoRune.ATProto.create_record(text: \"Hello world!\")\n\n# Is equivalent to:\nProtoRune.XRPC.procedure(session,\n  \"com.atproto.repo.createRecord\",\n  %{\n    repo: session.did,\n    collection: \"app.bsky.feed.post\",\n    record: %{text: \"Hello world!\"}\n  }\n)\n```","ref":"xrpc.html#xrpc-in-protorune"},{"type":"extras","title":"Using XRPC Directly - Understanding XRPC and Low-Level API Usage","doc":"","ref":"xrpc.html#using-xrpc-directly"},{"type":"extras","title":"Queries (GET Requests) - Understanding XRPC and Low-Level API Usage","doc":"```elixir\n# Get a profile\n{:ok, profile} = ProtoRune.XRPC.query(session,\n  \"app.bsky.actor.getProfile\",\n  %{actor: \"alice.bsky.social\"}\n)\n\n# List records with parameters\n{:ok, posts} = ProtoRune.XRPC.query(session,\n  \"app.bsky.feed.getAuthorFeed\",\n  %{\n    actor: \"bob.bsky.social\",\n    limit: 50,\n    filter: \"posts_with_media\"\n  }\n)\n```","ref":"xrpc.html#queries-get-requests"},{"type":"extras","title":"Procedures (POST Requests) - Understanding XRPC and Low-Level API Usage","doc":"```elixir\n# Create a record\n{:ok, record} = ProtoRune.XRPC.procedure(session,\n  \"com.atproto.repo.createRecord\",\n  %{\n    repo: session.did,\n    collection: \"app.bsky.feed.post\",\n    record: %{\n      text: \"Hello via XRPC!\",\n      createdAt: DateTime.utc_now() |> DateTime.to_iso8601()\n    }\n  }\n)\n\n# Delete a record\n{:ok, _} = ProtoRune.XRPC.procedure(session,\n  \"com.atproto.repo.deleteRecord\",\n  %{\n    repo: session.did,\n    collection: \"app.bsky.feed.post\",\n    rkey: \"1234\"\n  }\n)\n```","ref":"xrpc.html#procedures-post-requests"},{"type":"extras","title":"Error Handling - Understanding XRPC and Low-Level API Usage","doc":"XRPC provides structured errors:\n\n```elixir\ncase ProtoRune.XRPC.query(session, \"app.bsky.feed.getPost\", %{uri: invalid_uri}) do\n  {:ok, post} -> \n    # Handle success\n  \n  {:error, %ProtoRune.XRPC.Error{\n    code: :not_found,\n    message: \"Post not found\"\n  }} ->\n    # Handle specific error\n    \n  {:error, %ProtoRune.XRPC.Error{code: :rate_limit}} ->\n    # Handle rate limiting\nend\n```","ref":"xrpc.html#error-handling"},{"type":"extras","title":"Working with Lexicons - Understanding XRPC and Low-Level API Usage","doc":"XRPC endpoints are defined by Lexicons. ProtoRune generates code from these definitions:\n\n```elixir\n# Generated module for an XRPC method\ndefmodule ProtoRune.Lexicons.ATProto.Repo.CreateRecord do\n  @type params :: %{\n    repo: String.t(),\n    collection: String.t(),\n    rkey: String.t() | nil,\n    validate: boolean() | nil,\n    record: map()\n  }\n\n  @type response :: %{\n    uri: String.t(),\n    cid: String.t()\n  }\n  \n  def path, do: \"com.atproto.repo.createRecord\"\n  def method, do: :post\nend\n```","ref":"xrpc.html#working-with-lexicons"},{"type":"extras","title":"Custom XRPC Methods - Understanding XRPC and Low-Level API Usage","doc":"For methods not covered by ProtoRune's high-level API:\n\n```elixir\n# Define your method\ndefmodule MyApp.CustomMethod do\n  use ProtoRune.XRPC.Method,\n    path: \"com.example.customMethod\",\n    method: :post\n\n  @type params :: %{\n    customField: String.t()\n  }\n  \n  @type response :: %{\n    result: String.t()\n  }\nend\n\n# Use it\nProtoRune.XRPC.call(session, MyApp.CustomMethod, %{\n  customField: \"value\"\n})\n```","ref":"xrpc.html#custom-xrpc-methods"},{"type":"extras","title":"Advanced Usage - Understanding XRPC and Low-Level API Usage","doc":"","ref":"xrpc.html#advanced-usage"},{"type":"extras","title":"Raw Requests - Understanding XRPC and Low-Level API Usage","doc":"Access the underlying HTTP client:\n\n```elixir\n# Direct HTTP request\nProtoRune.XRPC.request(session,\n  method: :post,\n  path: \"com.atproto.repo.createRecord\",\n  body: data,\n  headers: [{\"Content-Type\", \"application/json\"}]\n)\n```","ref":"xrpc.html#raw-requests"},{"type":"extras","title":"Custom Response Handling - Understanding XRPC and Low-Level API Usage","doc":"Process raw responses:\n\n```elixir\ncase ProtoRune.XRPC.raw_query(session, \"app.bsky.feed.getTimeline\") do\n  {:ok, %{status: 200, body: body}} ->\n    # Handle raw response\n    \n  {:ok, %{status: status}} when status in 400..499 ->\n    # Handle client error\n    \n  {:error, _reason} ->\n    # Handle network error\nend\n```","ref":"xrpc.html#custom-response-handling"},{"type":"extras","title":"Best Practices - Understanding XRPC and Low-Level API Usage","doc":"1. **Use High-Level APIs First**: Only drop to XRPC when needed\n2. **Handle Rate Limits**: Implement exponential backoff\n3. **Validate Input**: Check params match Lexicon schemas\n4. **Type Everything**: Use typespecs for custom methods","ref":"xrpc.html#best-practices"},{"type":"extras","title":"Further Reading - Understanding XRPC and Low-Level API Usage","doc":"- [AT Protocol XRPC Spec](https://atproto.com/specs/xrpc)\n- [Lexicon Reference](https://atproto.com/specs/lexicon)\n- [XPRC HTTP Status Codes](https://atproto.com/specs/xrpc#summary-of-http-status-codes)","ref":"xrpc.html#further-reading"},{"type":"extras","title":"Bot Development with ProtoRune","doc":"# Bot Development with ProtoRune","ref":"bots.html"},{"type":"extras","title":"Creating a Bot - Bot Development with ProtoRune","doc":"A bot in ProtoRune is an OTP process that handles events from the ATProto network. \n\n```elixir\ndefmodule MyBot do\n  use ProtoRune.Bot,\n    name: :my_bot,\n    strategy: :polling,\n    service: \"https://bsky.social\" # defaults to the config service\n\n  @impl true\n  def get_identifier, do: System.fetch_env!(\"BOT_IDENTIFIER\")\n  def get_password, do: System.fetch_env!(\"BOT_PASSWORD\")\n  \n  @impl true\n  def handle_event(:like, %{uri: uri, user: user}) do\n    Logger.info(\"Got like from #{user.handle}\")\n  end\nend\n```","ref":"bots.html#creating-a-bot"},{"type":"extras","title":"Event Types - Bot Development with ProtoRune","doc":"Bots receive these event types:\n\n```elixir\n@type event ::\n  :like |    # Someone liked bot's post\n  :reply |   # Reply to bot's post\n  :mention | # Bot was mentioned\n  :repost |  # Bot's post was reposted\n  :quote |   # Bot's post was quoted\n  :follow    # Bot gained a follower\n```","ref":"bots.html#event-types"},{"type":"extras","title":"Event Payloads - Bot Development with ProtoRune","doc":"Each event receives relevant data:\n\n```elixir\n @type user :: %{\n   did: String.t(),\n   handle: String.t(), \n   display_name: String.t() | nil,\n   avatar_url: String.t() | nil\n }\n\n @type like_payload :: %{\n   uri: String.t(),          # URI of the liked post\n   user: user(),             # User who liked the post \n   post: ProtoRune.Bsky.Post.t(),  # The post that was liked\n   created_at: DateTime.t()   # When the like happened\n }\n\n @type reply_payload :: %{\n   uri: String.t(),          # URI of the reply post\n   user: user(),             # User who replied\n   post: ProtoRune.Bsky.Post.t(),  # The reply post\n   reply_to: ProtoRune.Bsky.Post.t(), # Original post being replied to\n   created_at: DateTime.t()   # When the reply happened\n }\n\n @type mention_payload :: %{\n   uri: String.t(),          # URI of post containing mention\n   user: user(),             # User who mentioned the bot\n   post: ProtoRune.Bsky.Post.t(),  # Post containing the mention\n   created_at: DateTime.t()   # When the mention happened\n }\n\n @type repost_payload :: %{\n   uri: String.t(),          # URI of the repost \n   user: user(),             # User who reposted\n   post: ProtoRune.Bsky.Post.t(),  # Original post that was reposted\n   created_at: DateTime.t()   # When the repost happened\n }\n\n @type quote_payload :: %{\n   uri: String.t(),          # URI of the quote post\n   user: user(),             # User who quoted\n   post: ProtoRune.Bsky.Post.t(),  # The quote post\n   quoted_post: ProtoRune.Bsky.Post.t(), # Original post being quoted\n   created_at: DateTime.t()   # When the quote happened\n }\n\n @type follow_payload :: %{\n   uri: String.t(),          # URI of the follow\n   user: user(),             # User who followed\n   created_at: DateTime.t()   # When the follow happened\n }\n```","ref":"bots.html#event-payloads"},{"type":"extras","title":"Strategies - Bot Development with ProtoRune","doc":"ProtoRune bots can use one of two strategies to receive events and notifications:\n\n1. Polling \n<!-- 2. Firehose (real-time subscription) -->\n\nLet's dive into each of these in more depth.","ref":"bots.html#strategies"},{"type":"extras","title":"Polling Strategy - Bot Development with ProtoRune","doc":"The polling strategy periodically checks the AT Proto notifications endpoint for new events and notifications. You can specify the polling interval in miliseconds. \n\nHere's an example of configuring a bot to use the polling strategy:\n\n```elixir\nuse ProtoRune.Bot,\n  strategy: :polling,\n  polling: %{\n    interval: :timer.seconds(30),     # Poll every 30 seconds\n    process_from: DateTime.utc_now()  # Start processing events from the current time\n  }\n```\n\nSome key things to understand about the polling strategy:\n\n- It allows you to control the frequency of checking for new events via the `interval` parameter. A shorter interval means more \"real-time\"-like behavior but also more requests to the server. A longer interval is easier on server resources but means a delay in processing events.\n\n- The `process_from` parameter lets you specify a start time for event processing. This is useful if you want to ignore old events when first starting the bot. By default it will process all historical events.\n\n- Polling is a simple approach to implement and reason about. However, for high volume bots or cases where minimizing latency is critical, the firehose strategy may be a better fit.\n\n- If the AT Proto server being used is experiencing issues or slow to respond, a polling bot will just keep retrying on its regular interval. Make sure your polling frequency isn't too aggressive.\n\n<!--","ref":"bots.html#polling-strategy"},{"type":"extras","title":"Firehose Strategy --> - Bot Development with ProtoRune","doc":"<!-- The firehose strategy opens a persistent connection to a relay server to receive a real-time stream of events as they occur. --> \n\n<!-- Here's how you would configure a firehose bot: -->\n\n<!-- ```elixir -->\n<!-- use ProtoRune.Bot, -->  \n  <!-- strategy: :firehose, -->\n  <!-- firehose: %{ -->\n    <!-- relay_uri: \"wss://bsky.network\", -->\n    <!-- filters: [\"app.bsky.feed.*\"], -->  \n    <!-- cursor: \"latest\" -->\n  <!-- } -->\n<!-- ``` -->\n\n<!-- Some important aspects of the firehose approach: -->\n\n<!-- - You specify the `relay_uri` of the server to connect to. This should be a WebSocket (`ws://` or `wss://`) URL. --> \n\n<!-- - The `filters` option lets you specify which types of events you want to receive. This uses a glob pattern matching the event schemas. For example `[\"app.bsky.feed.*\"]` will match all feed related events. Omitting `filters` or passing `[\"*\"]` will match all events. -->\n\n<!-- - The `cursor` specifies where in the event stream to start consuming from. By default it starts with the `\"latest\"` events. You can also pass a specific event sequence ID to start from. -->\n\n<!-- - Firehose provides events with lower latency than polling, since events are pushed immediately as they occur. This is ideal for bots that need to respond in real-time. -->\n\n<!-- - If the WebSocket connection is lost, the bot will automatically attempt to reconnect with exponential backoff. However, events may be missed in between reconnection attempts, so mission critical bots may want to implement additional resiliency. -->\n\n<!-- - Firehose can be more efficient than polling for high throughput, since a single connection is used. But it does require more server resources to maintain an open connection per bot. -->","ref":"bots.html#firehose-strategy"},{"type":"extras","title":"Which Strategy to Choose? - Bot Development with ProtoRune","doc":"The choice between polling and firehose depends on your bot's specific needs:\n\n- For bots that don't require immediate event processing and aim to be very simple, polling is a good choice. It's also a good starting point when developing a new bot.\n\n- For bots that need low latency processing, handle a high volume of events, or implement any real-time features, firehose is the way to go. Examples could be chat bots, moderation bots, or notification bots.\n\n<!-- - Consider the operational burden as well. Firehose bots are a bit more complex to run and monitor, due to the persistent connections. Polling bots are simpler but may generate a lot of requests. -->","ref":"bots.html#which-strategy-to-choose"},{"type":"extras","title":"Bot State - Bot Development with ProtoRune","doc":"Store state in the bot process:\n\n```elixir\ndefmodule StatefulBot do\n  use ProtoRune.Bot\n  \n  @impl true\n  def init(_opts) do\n    {:ok, %{replies: 0}}\n  end\n\n  @impl true\n  def handle_event(:reply, _payload, state) do\n    new_state = Map.update!(state, :replies, & &1 + 1)\n    {:ok, new_state}\n  end\nend\n```","ref":"bots.html#bot-state"},{"type":"extras","title":"Running Multiple Bots - Bot Development with ProtoRune","doc":"Each bot is a supervised process so you can manage multiple of them with a Supervisor:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      GreeterBot,\n      ModeratorBot,\n      AnalyticsBot\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```","ref":"bots.html#running-multiple-bots"},{"type":"extras","title":"Error Handling - Bot Development with ProtoRune","doc":"The bot will automatically:\n- Retry on rate limits with backoff\n- Refresh expired sessions\n- Reconnect on connection loss\n- Log errors\n\nHandle specific errors in events:\n\n```elixir\ndef handle_event(:reply, payload, state) do\n  case reply_to_mention(payload) do\n    {:ok, _} -> {:ok, state}\n    {:error, :rate_limited} -> {:retry, state} \n    {:error, _} = err -> err\n  end\nend\n```","ref":"bots.html#error-handling"},{"type":"extras","title":"Identity Management","doc":"# Identity Management\n\nTODO","ref":"identity.html"},{"type":"extras","title":"README","doc":"ProtoRune \n    A type-safe, well-documented AT Protocol SDK and bot framework for Elixir  \n \n\n> [!WARNING]\n>\n> This library is under active development and isn't production ready, expect breaking chnages","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"```elixir\ndef deps do\n  [\n    {:proto_rune, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - README","doc":"```elixir\n# Create a session\n{:ok, session} = ProtoRune.create_session(\"handle.bsky.social\", \"app-password\")\n\n# Post something\n{:ok, post} = ProtoRune.Client.create_post(session, \"Hello from Elixir!\")\n\n# Create a bot\ndefmodule MyBot do\n  use ProtoRune.Bot, name: :my_bot, strategy: :polling\n\n  @impl true\n  def handle_event(:like, %{uri: uri, user: user}) do\n    # Handle like event\n  end\nend\n\nMyBot.start_link()\n```","ref":"readme.html#quick-start"},{"type":"extras","title":"Examples - README","doc":"- [Simple bot with event handling](examples/simple_bot.ex)\n- [Post with rich text and embeds](examples/rich_post.ex) \n- [Custom feed generator](examples/feed_generator.ex)\n- [Firehose subscription](examples/firehose.ex)","ref":"readme.html#examples"},{"type":"extras","title":"Architecture - README","doc":"ProtoRune is organized into focused modules:\n\n- `ATProto` - Core protocol implementation (repo, identity, etc)\n- `Bsky` - Bluesky-specific features (feed, graph, notifications) \n- `Bot` - Bot framework with polling/firehose support\n- `XRPC` - Low-level XRPC client\n- `Lexicons` - Generated code from AT Protocol lexicons\n\n> Other submodules do exist like `ProtoRune.HTTPClient` but it are to be used internally","ref":"readme.html#architecture"},{"type":"extras","title":"Documentation - README","doc":"Full documentation is available at [hexdocs.pm/proto_rune](https://hexdocs.pm/proto_rune).\n\nThe guide covers:\n- [Getting Started](guides/getting_started.md)\n- [Bot Development](guides/bots.md)\n- [Working with Records](guides/records.md)\n- [XRPC Client Usage](guides/xrpc.md)\n- [Identity Management](guides/identity.md)","ref":"readme.html#documentation"},{"type":"extras","title":"Contributing - README","doc":"Pull requests welcome! See our [Contributing Guide](CONTRIBUTING.md).","ref":"readme.html#contributing"},{"type":"extras","title":"Inspirations - README","doc":"1. [Skyware](https://skyware.js.org/)\n2. [atcute](https://github.com/mary-ext/atcute)\n3. [Python AT Proto SDK](https://github.com/MarshalX/atproto)","ref":"readme.html#inspirations"},{"type":"extras","title":"License - README","doc":"MIT License - see [LICENSE](./LICENSE) for details.","ref":"readme.html#license"},{"type":"extras","title":"Contributing to ProtoRune","doc":"# Contributing to ProtoRune","ref":"contributing.html"},{"type":"extras","title":"Development Setup - Contributing to ProtoRune","doc":"1. Fork and clone the repository\n2. Install dependencies with `mix deps.get`\n3. Run tests with `mix test`","ref":"contributing.html#development-setup"},{"type":"extras","title":"Code Organization - Contributing to ProtoRune","doc":"```\nlib/proto_rune/\n├── atproto/      # Core AT Protocol \n├── bsky/         # Bluesky app features\n├── bot/          # Bot framework\n├── lexicons/     # Generated code\n└── xrpc/         # XRPC implementation\n```","ref":"contributing.html#code-organization"},{"type":"extras","title":"Code Style - Contributing to ProtoRune","doc":"- Run `mix format` before committing \n- Ensure 100% type coverage with dialyzer\n- Keep functions focused and small\n- Document public functions with `@doc` and `@moduledoc`\n- Add typespecs to all public functions","ref":"contributing.html#code-style"},{"type":"extras","title":"Testing - Contributing to ProtoRune","doc":"- Add tests for new features\n- Tests should be in `test/` mirroring `lib/` structure\n- Run full test suite with `mix test`\n- Run dialyzer with `mix dialyzer`","ref":"contributing.html#testing"},{"type":"extras","title":"Pull Requests - Contributing to ProtoRune","doc":"1. Create a branch from `main`\n2. Write descriptive commit messages \n3. Add tests for new functionality\n4. Update documentation as needed\n5. Submit PR with description of changes\n\nPRs should:\n- Have a clear purpose\n- Include relevant tests\n- Pass CI checks\n- Follow code style guidelines\n- Include documentation updates","ref":"contributing.html#pull-requests"},{"type":"extras","title":"Generated Code - Contributing to ProtoRune","doc":"The `lib/proto_rune/lexicons` directory contains generated code from AT Protocol lexicons. To regenerate:\n\n```bash\nmix gen_schemas --path priv/lexicons/ --output lib/proto_rune/lexicons/\n```\n\nDo not modify generated code directly. Update the generator instead.","ref":"contributing.html#generated-code"},{"type":"extras","title":"Release Process - Contributing to ProtoRune","doc":"1. Update version in `mix.exs`\n2. Update CHANGELOG.md\n3. Create GitHub release\n4. Publish to Hex.pm","ref":"contributing.html#release-process"},{"type":"extras","title":"Questions? - Contributing to ProtoRune","doc":"Open an issue or join the github repo discussion forum.","ref":"contributing.html#questions"},{"type":"extras","title":"LICENSE","doc":"Copyright 2024 zoedsoupe  \n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"RFC: ProtoRune Elixir SDK for AT Protocol","doc":"# RFC: ProtoRune Elixir SDK for AT Protocol","ref":"rfc.html"},{"type":"extras","title":"Abstract - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"This RFC proposes a domain-driven implementation of an AT Protocol SDK in Elixir, focusing on developer ergonomics, type safety, and scalable event processing. The implementation leverages Elixir's strengths in metaprogramming, concurrent processing, and functional design while providing clear boundaries between protocol layers.","ref":"rfc.html#abstract"},{"type":"extras","title":"Background - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"AT Protocol enables decentralized social networking through a layered architecture. While existing implementations like `indigo` and `skyware` provide foundational capabilities, an Elixir implementation can uniquely leverage BEAM's strengths for handling concurrent network operations and real-time event processing.","ref":"rfc.html#background"},{"type":"extras","title":"Goals - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"- Provide an intuitive, domain-driven API that reflects AT Protocol's layered architecture\n- Generate type-safe, validated code from Lexicons with comprehensive documentation\n- Enable efficient event processing through multiple strategies\n- Maintain extensibility for future AT Protocol features\n- Follow Elixir conventions and leverage BEAM capabilities","ref":"rfc.html#goals"},{"type":"extras","title":"Architecture - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"","ref":"rfc.html#architecture"},{"type":"extras","title":"Domain Organization - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"The codebase is organized by domain contexts at the root level, reflecting AT Protocol's layered architecture:\n\n```\nlib/\n  atproto/     # Core protocol implementation\n    xrpc/      # XRPC client and utilities\n    repo/      # Repository operations\n    identity/  # DID and handle resolution\n    sync/      # Data synchronization primitives\n    \n  lexicon/     # Generated code from Lexicons\n    app/\n      bsky/    # Bluesky-specific Lexicons\n    com/\n      atproto/ # Core protocol Lexicons\n      \n  bluesky/     # Bluesky application features\n  ozone/       # Content moderation features\n```","ref":"rfc.html#domain-organization"},{"type":"extras","title":"Code Generation - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"Lexicons are processed at compile-time through a mix task, generating Ecto schemas and type specifications:\n\n```elixir\ndefmodule Mix.Tasks.Compile.Lexicon do\n  use Mix.Task\n  \n  @impl Mix.Task\n  def run(_args) do\n    # Process lexicon JSON files\n    # Generate Elixir modules with proper namespacing\n  end\nend\n```\n\nThe generated code provides validation and documentation while maintaining clear mapping to AT Protocol data structures:\n\n```elixir\n# Abrevviated fields just for clarity and example purposes\ndefmodule Lexicon.App.Bsky.Feed.Post do\n  @moduledoc \"Record containing a Bluesky post.\"\n\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  @typedoc \"\"\"\n  - `text`: The primary post content. May be an empty string, if there are embeds.\n  - `created_at`: Client-declared timestamp when this post was originally created.\n  - `facets`: Annotations of text (mentions, URLs, hashtags, etc)\n  \"\"\"\n  @type t :: %__MODULE__{\n    text: String.t,\n    created_at: DateTime.t,\n    facets: list(Facet.t),\n  }\n  \n  @primary_key false\n  embedded_schema do\n    field :text, :string\n    field :created_at, :utc_datetime\n    \n    embeds_many :facets, Facet\n    embeds_one :embed, Embed\n  end\n  \n  def changeset(post, attrs) do\n    post\n    |> cast(attrs, [:text, :created_at])\n    |> validate_required([:text, :created_at])\n    |> validate_length(:text, max: 300)\n  end\nend\n```","ref":"rfc.html#code-generation"},{"type":"extras","title":"Error Handling - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"A unified error system categorizes and handles different types of failures:\n\n```elixir\ndefmodule ATProto.Error do\n  defexception [:type, :message, :reason]\n  \n  def transient(:rate_limit), do: %__MODULE__{\n    type: :transient,\n    message: \"Rate limited\",\n    reason: :rate_limit\n  }\n  \n  def permanent(:auth), do: %__MODULE__{\n    type: :permanent,\n    message: \"Authentication failed\",\n    reason: :auth\n  }\nend\n```","ref":"rfc.html#error-handling"},{"type":"extras","title":"Event Processing - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"The system supports multiple event processing strategies through a composable architecture:\n\n```elixir\ndefmodule ATProto.EventSource.Polling do\n  use GenServer\n  \n  def init(config) do\n    schedule_poll()\n    {:ok, %{interval: config.interval}}\n  end\n  \n  defp schedule_poll do\n    Process.send_after(self(), :poll, @interval)\n  end\nend\n\ndefmodule ATProto.EventSource.Firehose do\n  use GenServer\n  \n  def init(config) do\n    {:ok, socket} = connect_websocket(config)\n    {:ok, %{socket: socket}}\n  end\nend\n```\n\nBots can use any event source through a unified interface:\n\n```elixir\ndefmodule MyBot do\n  use ATProto.Bot,\n    name: :my_bot,\n    source: :firehose  # or :polling\n    \n  def handle_event(:like, payload) do\n    # Handle like event\n  end\nend\n```","ref":"rfc.html#event-processing"},{"type":"extras","title":"Supervision and Monitoring - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"Each bot runs under its own supervisor tree:\n\n```elixir\ndefmodule ATProto.Bot.Supervisor do\n  use Supervisor\n  \n  def init({bot_module, init_args}) do\n    children = [\n      {bot_module, init_args},\n      {ATProto.EventSource.Polling, init_args}\n    ]\n    \n    Supervisor.init(children, strategy: :one_for_all)\n  end\nend\n```\n\nTelemetry integration provides observability:\n\n```elixir\n:telemetry.span(\n  [:proto_rune, :polling],\n  %{bot: state.bot_name},\n  fn ->\n    result = fetch_notifications(state)\n    {result, %{duration: System.monotonic_time() - start_time}}\n  end\n)\n```","ref":"rfc.html#supervision-and-monitoring"},{"type":"extras","title":"Public API and Builder Patterns - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"","ref":"rfc.html#public-api-and-builder-patterns"},{"type":"extras","title":"Rich Content Construction - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"One of the most common tasks when working with AT Protocol is constructing rich text content with mentions, links, and formatting. We provide two complementary approaches: a pipeline-based builder pattern and a custom sigil.\n\nThe builder pattern leverages Elixir's pipe operator for readable, chainable operations:\n\n```elixir\ndefmodule ATProto.RichText do\n  @moduledoc \"\"\"\n  Provides a fluent API for constructing rich text content.\n  \n  The builder pattern maintains immutability while allowing\n  natural composition of text elements.\n  \"\"\"\n  \n  defstruct text: \"\", facets: []\n  \n  def new(initial_text \\\\ \"\") do\n    %__MODULE__{text: initial_text}\n  end\n  \n  def text(builder, content) do\n    %{builder | text: builder.text <> content}\n  end\n  \n  def mention(builder, handle) do\n    # Calculate byte indices for the mention\n    start = String.length(builder.text)\n    builder = text(builder, \"@#{handle}\")\n    \n    facet = %{\n      index: %{\n        byteStart: start,\n        byteEnd: start + byte_size(handle) + 1\n      },\n      features: [%{$type: \"app.bsky.richtext.facet#mention\", did: handle}]\n    }\n    \n    %{builder | facets: [facet | builder.facets]}\n  end\n  \n  # Similar implementations for links, hashtags, etc.\nend\n```\n\nThis can be used like:\n\n```elixir\n# Building rich text through method chaining\npost = ATProto.RichText.new()\n  |> ATProto.RichText.text(\"Hello \")\n  |> ATProto.RichText.mention(\"alice.bsky.social\")\n  |> ATProto.RichText.text(\"! Check out \")\n  |> ATProto.RichText.link(\"our project\", \"https://example.com\")\n  |> ATProto.RichText.hashtag(\"elixir\")\n```\n\nFor more extensive and complex use cases, we provide a custom sigil that offers a more concise syntax:\n\n```elixir\ndefmodule ATProto.Sigils do\n  @doc \"\"\"\n  Provides a markdown-like syntax for rich text construction.\n  \n  Supports:\n  - @mentions\n  - #hashtags\n  - [links](url)\n  \"\"\"\n  def sigil_f(text, _opts) do\n    # Parse the text and construct rich text with proper facets\n    ATProto.RichText.Parser.parse(text)\n  end\nend\n\n# Using the sigil\nimport ATProto.Sigils\n\n# parses the rich text on compile time ^-^\npost = ~f\"\"\"\nHello @alice.bsky.social!\nCheck out [our project](https://example.com) #elixir\n\"\"\"\n```","ref":"rfc.html#rich-content-construction"},{"type":"extras","title":"Post Creation and Interaction - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"The public API provides high-level functions for common operations while maintaining access to lower-level primitives:\n\n```elixir\ndefmodule ATProto.Bluesky do\n  @moduledoc \"\"\"\n  High-level API for Bluesky-specific operations.\n  \n  This module provides ergonomic functions for common tasks while\n  internally managing the complexities of AT Protocol interactions.\n  \"\"\"\n  \n  @type post_opts :: [\n    reply_to: String.t(),\n    langs: [String.t()],\n    labels: [String.t()],\n    # Other options...\n  ]\n  \n  @doc \"\"\"\n  Creates a new post with rich text content.","ref":"rfc.html#post-creation-and-interaction"},{"type":"extras","title":"Examples - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"# Simple text post\n      ATProto.Bluesky.post(session, \"Hello world!\")\n      \n      # Rich text with builder pattern\n      post_content = ATProto.RichText.new()\n        |> ATProto.RichText.text(\"Hello \")\n        |> ATProto.RichText.mention(\"alice.bsky.social\")\n      \n      ATProto.Bluesky.post(session, post_content)\n      \n      # Reply to another post\n      ATProto.Bluesky.post(session, \"Great point!\",\n        reply_to: \"at://did:plc:1234/app.bsky.feed.post/123\")\n  \"\"\"\n  @spec post(Session.t(), String.t() | RichText.t(), post_opts()) ::\n    {:ok, Post.t()} | {:error, Error.t()}\n  def post(session, content, opts \\\\ []) do\n    # Convert content to proper format\n    # Handle reply threading if reply_to is present\n    # Create post record\n    # Upload any embedded media\n    # Publish through XRPC\n  end\n  \n  @doc \"\"\"\n  Retrieves a thread of posts, handling pagination and\n  parent/child relationships.\n  \"\"\"\n  @spec get_thread(Session.t(), String.t(), keyword()) ::\n    {:ok, Thread.t()} | {:error, Error.t()}\n  def get_thread(session, uri, opts \\\\ []) do\n    # Fetch thread with proper depth\n    # Organize posts into thread structure\n    # Handle deleted/moderated content\n  end\nend\n```","ref":"rfc.html#examples"},{"type":"extras","title":"Repository Operations - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"For developers needing lower-level access, we expose the core repository operations while maintaining safety and proper error handling:\n\n```elixir\ndefmodule ATProto.Repo do\n  @moduledoc \"\"\"\n  Provides direct access to AT Protocol repository operations.\n  \n  These functions implement the foundational CRUD operations\n  defined by AT Protocol, with proper handling of CIDs, commits,\n  and Merkle tree validation.\n  \"\"\"\n  \n  @doc \"\"\"\n  Creates a record in a repository with proper validation\n  and Merkle tree updates.\n  \"\"\"\n  @spec create_record(Session.t(), String.t(), term(), keyword()) ::\n    {:ok, Record.t()} | {:error, Error.t()}\n  def create_record(session, collection, record, opts \\\\ []) do\n    with {:ok, validated} <- validate_record(collection, record),\n         {:ok, cid} <- compute_cid(validated),\n         {:ok, _} <- update_merkle_tree(session, cid, validated) do\n      # Commit changes\n    end\n  end\n  \n  @doc \"\"\"\n  Efficiently computes differences between two repository\n  states using Merkle Search Trees.\n  \"\"\"\n  @spec diff(repo_a :: String.t(), repo_b :: String.t()) ::\n    {:ok, [Record.t()]} | {:error, Error.t()}\n  def diff(repo_a, repo_b) do\n    # Use MST to identify different blocks\n    # Fetch only necessary records\n    # Return structured diff\n  end\nend\n```","ref":"rfc.html#repository-operations"},{"type":"extras","title":"Understanding Merkle Search Trees - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"The Merkle Search Tree implementation deserves special attention as it's fundamental to efficient repository synchronization. Here's a detailed look at its implementation:\n\n```elixir\ndefmodule ATProto.MST do\n  @moduledoc \"\"\"\n  Implements Merkle Search Trees for efficient repository\n  comparison and synchronization.\n  \n  MSTs combine the properties of:\n  - B-trees for efficient range queries\n  - Merkle trees for content verification\n  - Search trees for ordered key spaces\n  \"\"\"\n  \n  # Implementation details...\n  \n  @doc \"\"\"\n  Determines which blocks need to be synchronized between\n  two MSTs by comparing their structure.\n  \n  This is more efficient than comparing entire repositories\n  as it only needs to traverse branches that differ.\n  \"\"\"\n  def sync_blocks(local_root, remote_root) do\n    # Compare root hashes\n    # Traverse only differing branches\n    # Return minimal set of blocks needed\n  end\nend\n```","ref":"rfc.html#understanding-merkle-search-trees"},{"type":"extras","title":"Why These Design Choices? - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"1. **Builder Pattern**: We chose a builder pattern for rich text because it:\n   - Maintains immutability while being composable\n   - Provides clear, chainable operations\n   - Makes complex content construction readable\n   - Allows for extension with new content types\n\n2. **Custom Sigil**: The `~f` sigil complements the builder pattern by:\n   - Offering a concise syntax for simple cases\n   - Supporting familiar markdown-like formatting\n   - Making code more readable for text-heavy content\n\n3. **Layered API**: The API is structured in layers because:\n   - High-level functions handle common use cases simply\n   - Lower-level access enables advanced usage\n   - Domain separation maintains clear boundaries\n   - Each layer can evolve independently","ref":"rfc.html#why-these-design-choices"},{"type":"extras","title":"Implementation Phases - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"1. Core Protocol Layer (ATProto)\n   - XRPC client implementation\n   - Repository operations\n   - Identity resolution\n   - Basic synchronization\n\n2. Code Generation\n   - Lexicon parsing and IR\n   - Ecto schema generation\n   - Type specification generation\n   - Documentation generation\n\n3. Event Processing\n   - Polling implementation\n   - Firehose implementation\n   - Bot supervision\n   - Error handling\n\n4. Application Layer\n   - Bluesky integration\n   - Jetstream support\n   - Ozone integration","ref":"rfc.html#implementation-phases"},{"type":"extras","title":"Future Considerations - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"- Advanced event filtering and transformation\n- Custom Lexicon support\n- Integration with other AT Protocol applications (white-wind, ozone, teal, ...)","ref":"rfc.html#future-considerations"},{"type":"extras","title":"Security Considerations - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"- Rate limiting and backoff strategies\n- Secure credential management\n- Input validation and sanitization\n- Network timeout handling\n- Resource usage monitoring\n\nThe implementation must follow AT Protocol security guidelines and implement proper error handling for all network operations.","ref":"rfc.html#security-considerations"},{"type":"extras","title":"Conclusion - RFC: ProtoRune Elixir SDK for AT Protocol","doc":"This architecture provides a solid foundation for building AT Protocol applications in Elixir while maintaining extensibility for future protocol developments. The domain-driven organization will help developers build reliable applications while leveraging BEAM's strengths.\n\nThe proposed implementation balances developer ergonomics with protocol compliance, providing both high-level abstractions for common use cases and low-level access for advanced scenarios.","ref":"rfc.html#conclusion"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}